# encoding: utf-8


class Translator(object):
    """翻译器抽象类"""

    user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) " \
                 "Chrome/73.0.3683.86 Safari/537.36 "

    def __init__(self):
        self.trans_from = 'zh'
        self.trans_to = 'en'

    def get_result(self, *args, **kwargs):
        """获取翻译后的结果，类型如'字典(dict)'"""
        raise MethodNotImplementedException("subclass of AbstractRequest method get_result not implemented.")

    def query(self, *args, **kwargs):
        """单词或文本翻译接口
        @return self
        """
        return self

    def serialize(self, *args, **kwargs):
        """序列化"""
        raise MethodNotImplementedException("subclass of AbstractRequest method serialize not implemented.")

    def format(self, verbose=0):
        """格式化, 返回格式化的文本"""
        raise MethodNotImplementedException("subclass of AbstractRequest method view not implemented.")

    def set_trans_from(self, trans_from):
        """待翻译文字语种"""
        self.trans_from = trans_from

    def set_trans_to(self, trans_to):
        """翻译到哪个语种"""
        self.trans_to = trans_to

    def get_name(self):
        return ""


class MethodNotImplementedException(Exception):
    pass



from translate.util.baidu_response_format import *
from translate.util import get_plain_text


__all__ = ['BaiduFormatBuilder', 'YoudaoFormatBuilder']


class BaseFormatBuilder:
    def get_result(self) -> str: pass

    def __str__(self):
        return self.get_result()


class BaiduFormatBuilder(BaseFormatBuilder):
    def __init__(self):
        self.parts = {}

    def of_simple_means(self, simple):
        self.parts['simple'] = format_simple_means(simple)

        return self

    def of_oxford_entry(self, oxford):
        self.parts['oxford_entry'] = format_oxford_entry(oxford)

        return self

    def of_oxford_unbox(self, oxford):
        self.parts['ox_ford_unbox'] = format_oxford_unbox(oxford)
        return self

    def of_collins(self, collins):
        self.parts['collins'] = format_collins(collins)
        return self

    def of_double(self, double_data):
        self.parts['double'] = format_liju_double(double_data)
        return self

    def of_trans_result(self, trans_result):
        self.parts['trans_result'] = format_trans_result(trans_result)
        return self

    def _assembling(self, *keys, with_newline=True):
        result = ''
        for key in keys:
            if key in self.parts:
                value = self.parts[key]
                if with_newline:
                    value = with_new_line(value)
                result += value
        return result

    def get_result(self):
        return self._assembling('simple', 'oxford_entry', 'oxford_unbox', 'collins', 'double', 'trans_result')


class YoudaoFormatBuilder(BaseFormatBuilder):
    def __init__(self):
        self._parts = {}

    def of_phonetic_symbol(self, en, uk):
        res = ""
        if en:
            res += "    美式: \033[01m%s\033[0m\n" % en
        if uk:
            res += "    英式: \033[01m%s\033[0m\n" % uk

        if res:
            res = "发音:\n%s\n" % res

        self._parts['phonetic'] = res

        return self

    def of_english_chinese(self, data):
        if not data:
            self._parts['english_chinese'] = ''
            return self

        res = "英汉翻译:\n"
        trans1 = data['translation']
        for item in trans1:
            cont = ''
            if type(item) is dict:
                cont = item['content']
            else:
                cont = trans1[item]
            res += ("    " + cont + "\n")

        self._parts['english_chinese'] = res
        return self

    def of_web_interpretation(self, translation):
        if not translation:
            self._parts['interpretation'] = ''
            return self

        res = "\n网络释义:\n"
        for item in translation:
            if type(item) is not str:
                res += "    [%s]\n" % item['key']
            else:
                continue
            for t in item['trans']:
                if type(item['trans']) is list:
                    res += "        * %s\n" % get_plain_text(t['value'])
                else:
                    res += "        * %s\n" % get_plain_text(item['trans'][t])

        self._parts['interpretation'] = res
        return self

    def of_translation(self, translation):
        self._parts['translation'] = "%s\n" % get_plain_text(translation)
        return self

    def get_result(self):
        res = ''

        if 'translation' in self._parts:
            res += self._parts['translation']

        if 'phonetic' in self._parts:
            res += self._parts['phonetic']

        if 'english_chinese' in self._parts:
            res += self._parts['english_chinese']

        if 'interpretation' in self._parts:
            res += self._parts['interpretation']

        return res




import setuptools

from translate import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cmd_fanyi",
    version=__version__,
    author="raojinlin",
    author_email="1239015423@qq.com",
    description="command line translate tool",
    keywords="translate chinese english translate translator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/raojinlin/cmd-fanyi",
    packages=setuptools.find_packages(),
    scripts=[
        "bin/fanyi"
    ],
    install_requires=[
        'requests'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import json
import xml.etree.ElementTree as xmlElementTree

from xml.etree.ElementTree import ElementTree, Element


def is_list_child(tags, tag):
    """tag type
    :type tag str
    :type tags list
    :return:
    """
    i = 0
    for t in tags:
        if i > 1:
            return True
        if t == tag:
           i += 1 
    return i > 1


def append_attr_to_dict(attrib, result):
    """append xml element attrib to dict
    :param attrib: attributes
    :param result: dict
    :type result dict
    :type attrib dict
    :return: dict
    """
    for attr in attrib:
        result["@%s" % attr] = attrib[attr]
    return result


def _parse(node, result):
    """ parse xml tree to dict
    :param node:
    :type node Element
    :param result:
    :type result dict
    :return: dict
    """
    child_tag = []
    dict_child = []
    list_child = [] # filter(lambda tag: is_list_child(child_tag, tag), child_tag)

    for child in list(node):
        child_tag.append(child.tag)

        if len(list(child)) > 0:
            dict_child.append(child.tag)

    for tag in child_tag:
        if is_list_child(child_tag, tag):
            list_child.append(tag)

    if len(list(node)) == 0:
        node_text = node.text
        if node_text is None:
            node_text = append_attr_to_dict(node.attrib, {})
        result[node.tag] = node_text
        return result

    for child in list(node):
        if child.tag in list_child:
            if child.tag not in result:
                result[child.tag] = []
                append_attr_to_dict(child.attrib, result)

            result[child.tag].append(_parse(child, {}))
        elif child.tag in dict_child:
            result[child.tag] = {}
            append_attr_to_dict(child.attrib, result)

            _parse(child, result[child.tag])
        else:
            _parse(child, result)
    return result


def parser(xml):
    """parser warp
    :param xml:
    :type xml ElementTree
    :return:
    """
    return _parse(xml.getroot(), {})


if __name__ == '__main__':
    xml = xmlElementTree.parse("./test.xml")
    print(json.dumps(parser(xml)))

__all__ = ['parser']
